
#include "entropy.h"


double *returnEntropyList(equivalenceClass *eqClass, data *dataSet, configure *conf)
{
	double *returnValue;

	int noOfDim;

	int i, j;

	int **countDist;

	int value; 
	int noOfMember;

	int total;

	double currProb;

	equivalenceTuple **eqTuple;
	equivalenceTuple *eqOne;

	noOfDim = conf->noOfDim;
	eqTuple = eqClass->eqClass; 

	returnValue = (double *) malloc(sizeof(double)*noOfDim);
	memset(returnValue, 0, sizeof(double)*noOfDim);

	countDist = (int **) malloc(sizeof(int *)*noOfDim);
	for (i = 0; i < noOfDim; i++)
	{
		countDist[i] = (int *) malloc(sizeof(int)*dataSet->dimSize[i]);
		memset(countDist[i], 0, sizeof(int)*dataSet->dimSize[i]);
	}

	// count the distribution
	total = 0;
	for (i = 0; i < eqClass->noOfEqClass; i++)
	{

		//eqOne = &(eqTuple[i]);
		eqOne = eqTuple[i];
		noOfMember = eqOne->noOfMember;

		for (j = 0; j < noOfDim; j++)
		{
			value = eqOne->content[j].value;
			countDist[j][value] += noOfMember;
		}
		total += noOfMember;
	}

	// calculate the entropy
	for (i = 0; i < noOfDim; i++)
	{
		for (j = 0; j < dataSet->dimSize[i]; j++)
		{
			currProb = countDist[i][j]/(double)total;

			if (countDist[i][j] == 0) //(currProb == 0.0)
			{
				// do nothing
			}
			else
			{
				returnValue[i] += currProb*log(currProb);
			}
		}
		returnValue[i] *= -1;
	}

	// free
	for (i = 0; i < noOfDim; i++)
	{
		free(countDist[i]);
	}
	free(countDist);

	return returnValue;
}

int chooseBestAttribute(equivalenceClass *eqClass, data *dataSet, configure *conf)
{
	int returnValue;
	double maxValue;
	int noOfDim;

	double *entropyList;
	int isSen;

	int i, j;

	returnValue = 0;
	maxValue = 0.0;

	noOfDim = conf->noOfDim;

	entropyList = returnEntropyList(eqClass, dataSet, conf);

	for (i = 0; i < noOfDim; i++)
	{
		// check whether it is sensitive
		isSen = 0; 
		for (j = 0; j < conf->noOfSenAttr; j++)
		{
			if (conf->senAttrNo[j] == i)
			{
				isSen = 1;
				break;
			}
		}

		// if not sensitive attribute
		if (isSen == 0)
		{
			// check with the max value
			if (maxValue < entropyList[i])
			{
				maxValue = entropyList[i];
				returnValue = i;
			}
		}
	}

	free(entropyList);
	
	return returnValue;
}
